# Part 2: 파이썬 핵심 문법 마스터하기 (2주차)

**⬅️ 이전 시간: [Part 1: AI 개발 환경 완벽 구축 가이드](part_1_ai_development_environment.md)**

---

지난 시간에는 VSCode와 가상환경을 통해 우리만의 코딩 놀이터를 만들었습니다. 이제 모든 준비는 끝났습니다.
이번 주차에는 AI 개발의 가장 기본 언어인 파이썬의 핵심 문법들을 5일간의 여정을 통해 하나씩 정복하며, 실제 코드를 작성하는 즐거움을 느껴보는 시간을 갖겠습니다.

AI 개발의 세계에 오신 것을 환영합니다! 파이썬은 AI와 데이터 과학 분야에서 가장 사랑받는 언어입니다. 문법이 간결하고 사람의 생각과 비슷해서, 프로그래밍을 처음 시작하는 분들도 쉽게 배울 수 있습니다. 2주차에는 파이썬의 가장 핵심적인 문법을 하나씩 정복해나갑니다.

---

## 1일차(월): 모든 것의 시작, 변수와 자료형

### 🎯 오늘의 목표
- 변수(Variable)를 사용해 데이터를 저장하고 이름을 붙일 수 있습니다.
- 숫자(Integer, Float), 문자열(String), 불리언(Boolean) 등 파이썬의 기본 데이터 종류를 이해합니다.

---

### 1. 변수(Variable): 데이터에 이름표 붙이기
프로그래밍은 결국 '데이터'를 다루는 일입니다. <a href="glossary.md#variable">변수(Variable)</a>는 숫자, 글자 같은 데이터를 저장하는 '상자'입니다. 이 상자에 `name`, `age` 같은 이름표를 붙여두면, 필요할 때마다 이름표를 불러서 상자 안의 데이터를 쉽게 꺼내 쓸 수 있습니다.

#### 💡 쉽게 이해하기: 변수는 '이름이 붙은 저장 상자'입니다
> 여러분이 이사할 때, 짐을 담은 상자마다 '주방용품', '겨울옷', '책'처럼 내용물이 무엇인지 이름표(라벨)를 붙여놓습니다. 나중에 '책' 상자가 필요하면, 모든 상자를 열어볼 필요 없이 '책'이라는 이름표가 붙은 상자만 찾으면 되죠.
>
> 변수가 바로 이 **'이름표가 붙은 상자'**입니다. `age = 30` 이라는 코드는, `30`이라는 데이터를 `age`라는 이름의 상자에 넣는다는 뜻입니다. 이제부터 우리는 `age`라는 이름만 부르면 컴퓨터가 알아서 `30`이라는 값을 찾아줍니다.

> [!TIP]
> 이 파트의 모든 코드 예제는 아래 링크의 파이썬 파일에서 직접 실행하고 수정해볼 수 있습니다.
> **실행 가능한 소스 코드: [`part_2_3_python_syntax_collections.py`](../source_code/part_2_3_python_syntax_collections.py)**

```python
# 변수 만들기 (할당)
my_name = "파이" # 문자열 데이터
my_age = 20      # 숫자 (정수) 데이터
pi = 3.14        # 숫자 (실수) 데이터
is_student = True # 불리언 (참/거짓) 데이터

# 변수 사용하기 (참조)
print("안녕하세요, 제 이름은", my_name, "입니다.")
print("제 나이는", my_age, "살이고, 학생인가요?", is_student)
```

### 2. 기본 자료형(Data Types): 데이터의 종류
파이썬은 상자에 담는 데이터의 종류를 구분합니다.

- **정수 (Integer)**: `1`, `100`, `-5` 같이 소수점이 없는 숫자.
- **실수 (Float)**: `3.14`, `-0.5` 같이 소수점이 있는 숫자.
- **문자열 (String)**: `"안녕하세요"`, `'Python'` 같이 따옴표로 감싼 글자들.
- **<a href="glossary.md#boolean">불리언 (Boolean)</a>**: `True` 또는 `False`. 조건이 맞는지 틀리는지를 나타내는 값입니다.

---

### 💡 쉽게 이해하기: 파이썬의 '데이터 그릇'들
> 요리할 때 재료에 따라 다른 그릇을 쓰듯, 파이썬에서는 데이터의 종류와 사용 방식에 따라 다양한 '그릇'(자료구조)을 사용합니다. 앞으로 배울 4가지 핵심 그릇을 미리 맛봅시다.
>
> - **리스트(List) `[]`**: '쇼핑 목록'과 같습니다. 목록에 물건을 추가하거나, 빼거나, 순서를 바꿀 수 있습니다. 순서가 중요하고 내용이 계속 바뀔 때 사용합니다.
> - **튜플(Tuple) `()`**: '돌에 새긴 약속'과 같습니다. 한번 정해진 내용은 절대 바꿀 수 없습니다. (x, y) 좌표처럼 변하면 안 되는 중요한 값을 보관할 때 사용합니다.
> - **딕셔너리(Dictionary) `{}`**: '사전'이나 '전화번호부'입니다. '사과'라는 단어(Key)를 찾으면 'apple'이라는 뜻(Value)이 나오는 것처럼, 키를 통해 값을 빠르게 찾아냅니다. 순서보다는 의미를 통해 데이터를 관리할 때 씁니다.
> - **셋(Set) `{}`**: '주머니 속 구슬 모음'입니다. 같은 색깔의 구슬은 여러 번 넣어도 하나만 있는 것으로 칩니다 (중복 불가). 순서 없이 '어떤 구슬이 주머니 안에 있는가?'가 중요할 때 사용합니다.

## 2일차(화): 여러 데이터를 담는 그릇 (1) - 리스트와 튜플

### 🎯 오늘의 목표
- 순서가 있는 데이터 묶음인 리스트(List)를 다룰 수 있습니다.
- 변경이 불가능한 튜플(Tuple)의 특징과 사용 사례를 이해합니다.
- 리스트 컴프리헨션으로 파이썬다운 코드를 작성합니다.

---

### 1. 리스트(List): 순서가 있고 변경 가능한 만능 주머니
- 여러 개의 값을 순서대로 저장하는 **변경 가능한** <a href="glossary.md#list">자료구조</a>입니다.
- 대괄호 `[]`로 감싸고, 각 요소는 쉼표 `,`로 구분합니다.

```python
fruits = ["apple", "banana", "cherry"]

# 인덱싱과 슬라이싱 (문자열과 동일)
print(f"첫 번째 과일: {fruits[0]}")
print(f"마지막 두 개 과일: {fruits[1:]}")

# 리스트 값 변경하기
fruits[1] = "blueberry"
print(f"변경된 리스트: {fruits}")

# 리스트에 요소 추가/제거
fruits.append("strawberry") # 맨 뒤에 추가
print(f"추가 후: {fruits}")
fruits.pop() # 맨 뒤 요소 제거
print(f"제거 후: {fruits}")
print(f"리스트의 길이: {len(fruits)}")
```

### 2. 파이썬다운 코드: 리스트 컴프리헨션 (List Comprehension)
`for` 반복문과 `append`를 사용하여 리스트를 만드는 것을 한 줄로 간결하게 표현하는 방법입니다.

```python
# 0부터 9까지의 숫자 중 짝수만 제곱하여 리스트 만들기

# 일반적인 for문 사용
squares = []
for i in range(10):
    if i % 2 == 0:
        squares.append(i**2)
print(f"for문 사용: {squares}")

# 리스트 컴프리헨션 사용 (훨씬 간결!)
squares_comp = [i**2 for i in range(10) if i % 2 == 0]
print(f"컴프리헨션 사용: {squares_comp}")
```
<a href="glossary.md#list-comprehension">리스트 컴프리헨션</a>에 익숙해지면 코드가 훨씬 깔끔해지고, 작성 속도도 빨라집니다.

#### 💡 쉽게 이해하기: 리스트 컴프리헨션은 '주문 제작 기계'입니다
> 리스트 컴프리헨션은 마치 '주문 제작 기계'와 같습니다. `for`문을 사용하는 것은 재료(숫자)를 하나씩 손으로 가공해서(제곱해서) 상자에 담는 수작업과 비슷합니다.
>
> 반면, 리스트 컴프리헨션은 이 기계에 **`[결과물(i**2) | 재료(for i in range(10)) | 조건(if i % 2 == 0)]`** 이라는 설계도를 한 번에 넣어주는 것과 같습니다. 그러면 기계가 알아서 설계도에 맞는 제품들만 순식간에 만들어내어 컨테이너 벨트 위에 올려줍니다. 훨씬 빠르고, 간결하며, 세련된 방식이죠!

### 3. 튜플(Tuple): 순서가 있지만 변경 불가능한 금고
- 여러 개의 값을 순서대로 저장하는 **변경 불가능한** <a href="glossary.md#tuple">자료구조</a>입니다.
- 소괄호 `()`로 감싸서 만듭니다. 한번 만들어지면 내용을 바꿀 수 없어서, 안정성이 중요할 때 사용됩니다.

```python
point = (10, 20)
print(f"x좌표: {point[0]}")

# 튜플은 변경 불가! 아래 코드는 오류(TypeError)를 발생시킵니다.
# point[0] = 15
```
> **언제 튜플을 사용할까?**
> - 함수가 여러 개의 값을 반환할 때
> - <a href="glossary.md#dictionary">딕셔너리</a>의 키(Key) 값으로 사용해야 할 때
> - 프로그램 전체에서 변하면 안 되는 상수 값을 정의할 때

---

## 3일차(수): 여러 데이터를 담는 그릇 (2) - 딕셔너리와 셋

### 🎯 오늘의 목표
- `Key:Value` 쌍으로 데이터를 저장하는 딕셔너리(Dictionary)를 다룰 수 있습니다.
- 중복을 허용하지 않는 셋(Set)의 특징과 집합 연산을 이해합니다.

---

### 1. 딕셔너리(Dictionary): 의미를 부여한 데이터 관리법
- `Key:Value` 쌍으로 데이터를 저장하며, 순서가 중요하지 않습니다.
- 중괄호 `{}`로 만듭니다. '이름'='홍길동', '나이'=30 처럼 의미를 가진 <a href="glossary.md#dictionary">딕셔너리</a> 데이터를 관리할 때 매우 유용합니다.

```python
user = {
    "name": "앨리스",
    "age": 30,
    "city": "New York"
}

# Key를 이용해 Value에 접근
print(f"이름: {user['name']}")
print(f"나이 (안전하게): {user.get('age')}") # Key가 없어도 오류 대신 None 반환

# 데이터 추가 및 변경
user['email'] = "alice@example.com"
user['age'] = 31
print(f"수정된 정보: {user}")

# 모든 Key와 Value 보기
print(f"키 목록: {list(user.keys())}")
print(f"값 목록: {list(user.values())}")
```

### 2. 셋(Set): 중복을 제거하고 관계를 파악하는 도구
- **중복을 허용하지 않는** 순서 없는 데이터의 모음입니다.
- 주로 데이터의 중복을 제거하거나, 여러 데이터 그룹 간의 연산(합집합, 교집합 등)에 사용되는 <a href="glossary.md#set">셋</a>입니다.

```python
unique_numbers = {1, 2, 3, 2, 1, 4}
print(f"중복이 제거된 집합: {unique_numbers}") # {1, 2, 3, 4}

set_a = {1, 2, 3, 4}
set_b = {3, 4, 5, 6}

# 집합 연산
print(f"합집합 (A | B): {set_a | set_b}")
print(f"교집합 (A & B): {set_a & set_b}")
print(f"차집합 (A - B): {set_a - set_b}")
```

---

## 4일차(목): 코드의 흐름 제어하기 - 제어문

### 🎯 오늘의 목표
- `if`문으로 조건에 따라 다른 코드를 실행할 수 있습니다.
- `for`문과 `while`문으로 특정 코드를 반복 실행할 수 있습니다.
- `enumerate`를 사용하여 리스트의 인덱스와 값을 함께 활용합니다.

---

### 1. 파이썬의 상징: 들여쓰기(Indentation)
파이썬은 **들여쓰기(indentation)**로 코드 블록을 구분합니다. 보통 4개의 공백(스페이스바 4번)을 사용하며, 이는 파이썬 문법의 **필수 요소**이므로 매우 중요합니다.

#### 💡 쉽게 이해하기: 들여쓰기는 '글의 목차'와 같습니다
> 우리가 보고서를 쓸 때, 큰 제목 아래에 소제목을 쓰고, 그 아래에 내용을 쓰는 것처럼 계층을 만듭니다. 이 계층 구조 덕분에 어떤 내용이 어떤 제목에 속하는지 한눈에 알 수 있습니다.
>
> 파이썬의 들여쓰기도 똑같습니다. `if`문이나 `for`문 아래에 오는 코드들을 안으로 들여쓰면, '이 코드들은 바로 위 `if`문(또는 `for`문)에 소속된 한 팀입니다'라고 알려주는 시각적인 신호입니다. 이 규칙만 잘 지키면 코드의 구조가 명확해지고, 다른 사람이 내 코드를 이해하기도 쉬워집니다. 파이썬에서는 이 규칙이 선택이 아닌 **필수**입니다!

### 2. `if-elif-else`: 조건에 따라 움직이기

```python
score = 85
if score >= 90:
    grade = "A"
elif score >= 80:
    grade = "B"
else:
    grade = "F"
print(f"당신의 학점은 {grade}입니다.")
```

### 3. `for`: 순서대로 반복하기
<a href="glossary.md#list">리스트</a>, <a href="glossary.md#tuple">튜플</a>, 문자열 등 순회 가능한(iterable) 객체의 요소를 하나씩 순회하며 코드를 반복합니다. 마치 **체크리스트의 항목을 하나씩 확인**하는 것과 같습니다.

```python
fruits = ["apple", "banana", "cherry"]

# 가장 기본적인 for문
for fruit in fruits:
    print(f"{fruit}를 먹었습니다.")

# enumerate를 사용하면 인덱스 번호와 값을 함께 사용할 수 있습니다.
for idx, fruit in enumerate(fruits):
    print(f"{idx+1}번째 과일은 {fruit}입니다.")

# range()를 사용하면 정해진 횟수만큼 반복할 수 있습니다.
for i in range(5): # 0부터 4까지 5번 반복
    print(f"{i}번째 반복")
```

### 4. `while`: 조건이 만족하는 동안 반복하기
주어진 조건이 `True`인 동안 코드를 계속해서 반복 실행합니다. **생명(life)이 0이 될 때까지 계속되는 게임**과 같습니다. 매 라운드 시작 전에 생명이 남아있는지(`life > 0`)를 확인하는 것과 같죠.

```python
count = 5
while count > 0:
    print(f"카운트 다운: {count}")
    count -= 1 # count를 1씩 감소시킵니다. 이 줄이 없으면 무한 루프에 빠집니다!

print("발사!")
```

---

## 5일차(금): 코드를 재사용하는 마법, 함수

### 🎯 오늘의 목표
- <a href="glossary.md#function">함수(Function)</a>를 정의하고 호출하여 코드의 재사용성을 높일 수 있습니다.
- 함수의 입력(매개변수)과 출력(반환 값)의 개념을 이해합니다.
- 반복되는 코드를 묶어 함수(Function)로 만들 수 있습니다.
- 함수에 입력을 주는 매개변수(Parameter)와 결과를 돌려받는 반환 값(Return)을 이해합니다.

---

### 1. 함수(Function): 나만의 미니 프로그램
코드를 짜다 보면, 비슷한 작업을 여러 번 반복해야 할 때가 있습니다. 함수는 특정 작업을 수행하는 코드 묶음에 이름을 붙여놓고, 필요할 때마다 이름만 불러서 사용하는 **'재사용 가능한 코드 조각'**입니다.

#### 💡 쉽게 이해하기: 함수는 '나만의 레시피' 또는 '미니 로봇'입니다
> 여러분이 '계란 후라이'를 만드는 방법을 '계란 후라이 레시피'라는 이름으로 정리해두었다고 상상해보세요. 앞으로 계란 후라이가 필요할 때마다, 매번 "기름을 두르고, 계란을 깨서 넣고, 소금을 뿌린다..."라고 생각할 필요 없이 그냥 "계란 후라이 레시피대로 해!"라고 말하면 됩니다.
>
> 함수가 바로 이 **'레시피'**와 같습니다. 특정 기능을 수행하는 코드들을 `def make_fried_egg():` 처럼 하나의 레시피로 만들어두면, 필요할 때마다 `make_fried_egg()`라고 호출만 하면 되는 것이죠. 코드가 훨씬 간결해지고, 관리하기 쉬워집니다.

### 2. 함수 만들기 (`def`) 와 사용하기 (호출)

`def` 키워드를 사용하여 함수를 정의(define)합니다.

```python
# 1. 함수 정의 (레시피 만들기)
def say_hello():
    print("안녕하세요!")
    print("파이썬의 세계에 오신 것을 환영합니다.")

# 2. 함수 호출 (레시피 사용하기)
say_hello()
say_hello()
```

### 3. 입력(매개변수)과 출력(반환 값)이 있는 함수
진짜 레시피처럼, 함수도 재료를 받아(입력) 요리를 완성해서(출력) 내놓을 수 있습니다.

- **매개변수 (Parameter)**: 함수에 전달하는 재료(입력 값).
- **반환 값 (Return)**: 함수가 작업을 마친 후 돌려주는 결과물(출력 값).

```python
# '이름'을 재료로 받아 인사말을 만들어주는 레시피
def make_greeting(name):
    greeting = f"안녕하세요, {name}님!"
    return greeting # 결과물(요리)을 반환

# 함수에 '앨리스'라는 재료를 넣어 호출하고, 결과물을 변수에 저장
alice_greeting = make_greeting("앨리스")
print(alice_greeting) # "안녕하세요, 앨리스님!" 출력

bob_greeting = make_greeting("밥")
print(bob_greeting) # "안녕하세요, 밥님!" 출력
```
함수를 잘 사용하면 복잡한 문제를 여러 개의 간단한 함수로 나누어 해결할 수 있어, 훨씬 체계적인 프로그래밍이 가능해집니다.

### 📝 2주차 요약

이번 주차에 우리는 파이썬의 가장 기본적인 구성 요소들을 배웠습니다.

- **변수와 자료형**: 데이터를 저장하고(변수), 데이터의 종류(숫자, 문자열, 불리언)를 구분하는 법을 배웠습니다.
- **자료구조**: 여러 데이터를 효율적으로 담는 그릇인 리스트, 튜플, 딕셔너리, 셋의 특징과 사용법을 익혔습니다.
- **제어문**: 코드의 흐름을 제어하는 `if`, `for`, `while` 문을 통해 프로그램이 특정 조건이나 반복에 따라 동작하게 만들었습니다.
- **함수**: 반복되는 코드를 `def` 키워드를 사용해 재사용 가능한 블록으로 만드는 방법을 배웠습니다.

이 핵심 문법들은 앞으로 우리가 만들 모든 AI 서비스의 뼈대가 될 것입니다. 각 개념이 손에 익을 때까지 여러 번 연습해보는 것이 중요합니다.

### 🔍 2주차 연습 문제

**문제 1: 학점 계산기 함수 만들기**
- 학생의 점수(0~100)를 입력받아 학점을 반환하는 `get_grade` 함수를 만들어보세요.
  - 90점 이상: "A"
  - 80점 이상 90점 미만: "B"
  - 70점 이상 80점 미만: "C"
  - 70점 미만: "F"

**문제 2: 리스트에서 특정 단어 개수 세기**
- 과일 이름들이 담긴 리스트 `fruits = ["apple", "banana", "cherry", "apple", "cherry", "apple"]`가 주어졌을 때, "apple"이 몇 번 나타나는지 `for`문을 사용해 세어보세요.

**문제 3: 구구단 출력 함수**
- 숫자를 하나 입력받아 해당 숫자의 구구단을 출력하는 `multiplication_table` 함수를 `while`문을 사용하여 만들어보세요.

---

➡️ **다음 시간: [Part 3: 파이썬 컬렉션, 더 깊게 이해하기](part_3_python_collections.md)**